@@ -2,6 +2,10 @@
 require 'timeout'
 require 'open-uri'
 class CartController < BaseController
+  include CartControllerExtend
+
+  ssl_required :temporary_shipping, :shipping, :delivery, :delivery2, :purchase, :purchase2, :purchase_confirm, :confirm, :complete, :before_finish, :finish, :select_delivery_time, :select_delivery_time_with_delivery_trader_id_ajax
+
   before_filter :cart_check, :only => [:temporary_shipping,:shipping, :purchase,:purchase2, :confirm, :complete, :delivery, :delivery2]
   before_filter :login_divaricate ,:only =>[:purchase,:purchase2,:confirm, :complete, :delivery, :delivery2]
   before_filter :login_check, :only => [:shipping]
@@ -15,7 +19,11 @@ class CartController < BaseController
   # カートの中を見る。Loginの可否、カート内容の有無で動的に変動。カート操作全般はここから行う。
   def show
     unless @carts.all?(&:valid?)
-      flash.now[:error] = cart_errors(@carts)
+      if flash.now[:error]
+        flash.now[:error] = flash.now[:error] + cart_errors(@carts)
+      else
+        flash.now[:error] = cart_errors(@carts)
+      end
     end
     @cart_point = total_points
     if @carts.last
@@ -221,6 +229,11 @@ class CartController < BaseController
     end
     @delivery_time_options = Hash.new
     @order_deliveries.each do |retailer_id, od|
+      if od.delivery_trader_id.blank?
+        flash.now[:error] = "発送方法が選択されていません"
+        delivery
+        return
+      end
       delivery_trader_id = od.delivery_trader_id
       option = select_delivery_time_with_delivery_trader_id(delivery_trader_id)
       @delivery_time_options[retailer_id] = option
@@ -238,7 +251,24 @@ class CartController < BaseController
     else
       #error
     end
-   
+
+    @order_deliveries.each do |key, value|
+      if value.delivery_trader_id.blank? 
+        flash.now[:error] = "発送方法が選択されていません"
+      elsif value.delivery_time_id.blank?
+        flash.now[:error] = "配達時間が選択されていません"
+      end
+      if flash.now[:error]
+        if request.mobile?
+          delivery2
+        else
+          params[:back] = "1"
+          delivery
+        end
+        return
+      end
+    end
+
     if params[:back] == "1"
       @payment_id = @order_deliveries.first[1].payment_id
     end
@@ -313,6 +343,11 @@ class CartController < BaseController
   #確認画面へ
   def confirm
     init_order_deliveries
+    if params[:order_delivery].nil? || params[:order_delivery][:payment_id].blank?
+      flash.now[:error] = "支払い方法が選択されていません"
+      render :action => 'purchase'
+      return
+    end
     @order_deliveries.each do |key, od|
       unless od.valid?
         render :action => 'purchase'
@@ -323,7 +358,7 @@ class CartController < BaseController
     if @login_customer
       @all_use_point = 0
       @order_deliveries.each do |retailer_id, od|
-        if params[:points][retailer_id][:point_check] == "true"
+        if !params[:points].nil? && !params[:points][retailer_id].nil? && params[:points][retailer_id][:point_check] == "true"
           use_point = od.use_point.to_i
           if use_point <= 0
             flash.now[:error] = '使用ポイントをご入力ください。'
@@ -349,7 +384,7 @@ class CartController < BaseController
       end
       @cart_point = total_points
       @point_after_operation = @login_customer.point.to_i - @all_use_point + @cart_point
-      
+      session[:point_after_operation] = @point_after_operation
     end
     
     @payment_total = 0
@@ -369,11 +404,11 @@ class CartController < BaseController
       redirect_to :action => :show
       return
     end
-    @login_customer.point = params[:point_after_operation] if @login_customer
+    @login_customer.point = session[:point_after_operation] if @login_customer
     @orders = Hash.new
     @order_deliveries = Hash.new
     @order_details = Hash.new
-    ids = Array.new
+    @ids = Array.new
     params[:order_deliveries].each do |key, _od|
       order = nil
       if @not_login_customer
@@ -392,7 +427,7 @@ class CartController < BaseController
       @order_details[key] = od.details_build_from_carts(cart)
       od.calculate_charge!
       od.calculate_total!
-      ids << @order_details[key].map{|o_d| o_d.product_style.product_id}
+      @ids << @order_details[key].map{|o_d| o_d.product_style.product_id}
     end  
 
     @order_deliveries.each do |key, od|
@@ -407,57 +442,42 @@ class CartController < BaseController
       return
     end
 
-    begin
-      Order.transaction do
-        @carts.each do | cart |
-          if request.mobile?
-            ProductAccessLog.create(:product_id => cart.product_style.product_id,
-                                    :session_id => session.session_id,
-                                    :customer_id => @login_customer && @login_customer.id,
-                                    :docomo_flg => request.mobile == Jpmobile::Mobile::Docomo,
-                                    :ident => request.mobile.ident,
-                                   :complete_flg => true)
-          end
-          product_style = ProductStyle.find(cart.product_style_id, :lock=>true)
-          product_style.order(cart.quantity)
-          product_style.save!
-          #会員のみキャンペーン処理
-          if @login_customer
-            cart.campaign_id and process_campaign(cart, @login_customer)  
-          end
-        end
-        # 非会員購入対応
-        if @login_customer
-          @login_customer.carts.delete_all
-          @login_customer.save!
-        end
-        
-        order_ids = Hash.new
-        @orders.each do |key, order|
-          order.save!
-          order_ids[key] = order.id
-          Notifier::deliver_buying_complete(order)
-        end
-        flash[:completed] = true
-        flash[:order_ids] = order_ids
-        flash[:googleanalytics_ecs] = add_googleanalytics_ecs(@orders, @order_deliveries, @order_details)
-        @carts.clear
-      end
-    rescue => e
+    # paymentロジックをプラグイン化するため、予めセッションに保存しておき画面遷移で引き回さないようにする
+    save_transaction_items_before_payment
+    payment_id =  @order_deliveries.first[1].payment_id
+    payment = Payment.find(payment_id)
+    #p "plugin_id: " + payment.payment_plugin_id.to_s
+    payment_plugin = payment.get_plugin_instance
+    #p payment_plugin.name
+    self.send payment_plugin.next_step(current_method_symbol)
+  end
+  def before_finish
+    unless restore_transaction_items_after_payment
       flash.now[:error] = '失敗しました'
+      redirect_to :action => 'show'
+      return
+    end
+    begin
+      save_before_finish
+    rescue => e
+      flash.now[:error] = 'データの保存に失敗しました。商品の在庫が切れた可能性があります。'
       logger.error(e.message)
       e.backtrace.each{|s|logger.error(s)}
       redirect_to :action => 'show'
       return
     end
-    redirect_to :action => :finish, :ids => ids
+    redirect_to :action => :finish, :ids => @ids
+    
   end
 
+
   def finish
     unless flash[:completed]
       render :template => 'cart/405', :status => :method_not_allowed
       return
     end
+    session[:point_after_operation] = nil
+    session[:transaction_items] = nil
     @recommend_buys = Recommend.recommend_get(params[:ids][0], Recommend::TYPE_BUY)
     @shop = Shop.find(:first)
     render :action => 'complete'
@@ -544,6 +564,45 @@ class CartController < BaseController
 
   private
 
+  def save_before_finish
+    Order.transaction do
+      @carts.each do | cart |
+        if request.mobile?
+          ProductAccessLog.create(:product_id => cart.product_style.product_id,
+                                  :session_id => session.session_id,
+                                  :customer_id => @login_customer && @login_customer.id,
+                                  :docomo_flg => request.mobile == Jpmobile::Mobile::Docomo,
+                                  :ident => request.mobile.ident,
+                                  :complete_flg => true)
+        end
+        product_style = ProductStyle.find(cart.product_style_id, :lock=>true)
+        product_style.order(cart.quantity)
+        product_style.save!
+        #会員のみキャンペーン処理
+        if @login_customer
+          cart.campaign_id and process_campaign(cart, @login_customer)  
+        end
+      end
+      # 非会員購入対応
+      if @login_customer
+        @login_customer.carts.delete_all
+        @login_customer.save!
+      end
+      
+      order_ids = Hash.new
+      @orders.each do |key, order|
+        order.save!
+        order_ids[key] = order.id
+        Notifier::deliver_buying_complete(order)
+      end
+      flash[:completed] = true
+      flash[:order_ids] = order_ids
+      flash[:googleanalytics_ecs] = add_googleanalytics_ecs(@orders, @order_deliveries, @order_details)
+      @carts.clear
+    end
+  end
+
+
 =begin rdoc
   * INFO
 
@@ -658,7 +717,7 @@ class CartController < BaseController
     params[:order_deliveries].each do |key, order_delivery|
       @order_deliveries[key] = OrderDelivery.new(order_delivery)
       @order_deliveries[key].set_customer(@login_customer) if @login_customer
-      @order_deliveries[key].payment_id = params[:order_delivery][:payment_id]
+      @order_deliveries[key].payment_id = params[:order_delivery][:payment_id] unless params[:order_delivery].nil?
     end
     @order_details_map = Hash.new
     @order_deliveries.each do |key, order_delivery|
@@ -781,5 +840,34 @@ class CartController < BaseController
     end)
     return options
   end
+  
+  def current_method_symbol
+    caller.first.scan(/`(.*)'/).to_s.intern
+  end
 
+  def save_transaction_items_before_payment
+    transaction_items = Hash.new
+    transaction_items[:carts] = @carts
+    transaction_items[:login_customer] = @login_customer
+    transaction_items[:orders] = @orders
+    transaction_items[:order_deliveries] = @order_deliveries
+    transaction_items[:order_details] = @order_details
+    transaction_items[:ids] = @ids
+    transaction_items[:not_login_customer] = @not_login_customer
+    session[:transaction_items] = transaction_items
+  end
+
+  def restore_transaction_items_after_payment
+    transaction_items = session[:transaction_items]
+    return false if transaction_items.nil?
+    @carts = transaction_items[:carts]
+    @login_customer = transaction_items[:login_customer]
+    @orders = transaction_items[:orders]
+    @order_deliveries = transaction_items[:order_deliveries]
+    @order_details = transaction_items[:order_details]
+    @ids = transaction_items[:ids]
+    @not_login_customer = transaction_items[:not_login_customer]
+    return true
+  end
+  
 end
